import { transform } from 'babel-core';
import * as babylon from 'babylon';
import { parse, print } from 'recast';
export class Source {
    constructor(path, content) {
        this.path = path;
        this.content = content;
    }
}
export class SourceTransformResult {
    constructor(source, output, error) {
        this.source = source;
        this.output = output;
        this.error = error;
    }
}
export default class TransformRunner {
    constructor(sources, plugins, delegate = {}) {
        this.sources = sources;
        this.plugins = plugins;
        this.delegate = delegate;
    }
    *run() {
        if (this.delegate.transformStart) {
            this.delegate.transformStart(this);
        }
        for (let source of this.sources) {
            if (this.delegate.transformSourceStart) {
                this.delegate.transformSourceStart(this, source);
            }
            let transformed;
            try {
                let output = this.transformSource(source);
                transformed = new SourceTransformResult(source, output, null);
            }
            catch (err) {
                transformed = new SourceTransformResult(source, null, err);
            }
            yield transformed;
            if (this.delegate.transformSourceEnd) {
                this.delegate.transformSourceEnd(this, transformed);
            }
        }
        if (this.delegate && this.delegate.transformEnd) {
            this.delegate.transformEnd(this);
        }
    }
    transformSource(source) {
        return transform(source.content, {
            parserOpts: {
                parser(code) {
                    return parse(code, {
                        parser: {
                            parse(code) {
                                return babylon.parse(code, {
                                    sourceType: 'module',
                                    allowImportExportEverywhere: false,
                                    allowReturnOutsideFunction: true,
                                    allowSuperOutsideMethod: true,
                                    plugins: [
                                        "flow",
                                        "jsx",
                                        "asyncGenerators",
                                        "classProperties",
                                        "doExpressions",
                                        "exportExtensions",
                                        "functionBind",
                                        "functionSent",
                                        "objectRestSpread",
                                        "dynamicImport"
                                    ]
                                });
                            }
                        }
                    });
                }
            },
            generatorOpts: {
                generator: print
            },
            plugins: this.plugins
        }).code;
    }
}
